import { IconButton } from '@suid/material';
import SearchIcon from '@suid/icons-material/Search';

import { Component, Show } from 'solid-js';
import { findAddress, getVillageOrTown } from '../../lib/osm';
import { cloneDeep } from 'lodash';
import { useI18n } from '@solid-primitives/i18n';
import dispatch from '../../workers/dispatcher-main';

interface Props {
  wpt: () => Wpt;
  dispatchParams: (wpt: Wpt) => { action: string; params: any };
}

const DisplayOrGetAddress: Component<Props> = ({ wpt, dispatchParams }) => {
  const [t, { add, locale, dict }] = useI18n();

  const findClickHandler = async () => {
    console.log({ caller: 'findClickHandler', wpt: wpt() });
    const address = await findAddress(wpt().$.lon, wpt().$.lat, locale);
    const updatedTarget = cloneDeep(wpt());
    if (updatedTarget.extensions === undefined) {
      updatedTarget.extensions = {};
    }
    updatedTarget.extensions.address = address;
    dispatch(dispatchParams(updatedTarget));
  };

  return (
    <Show
      when={wpt()?.extensions?.address}
      fallback={
        <IconButton color='primary' onClick={findClickHandler}>
          <SearchIcon />
        </IconButton>
      }
    >
      <>{getVillageOrTown(wpt()?.extensions?.address)}</>
    </Show>
  );
};

export default DisplayOrGetAddress;
