import _ from 'lodash';
import PouchDB from 'pouchdb';
import indexeddb from 'pouchdb-adapter-indexeddb';
import { openDatabases } from '../db-admin/open';
import uri from '../lib/ids';
import { toHex } from '../lib/to-hex';
import {
  getAccountById,
  getAccounts,
  initialAccount,
  putAccount,
} from './account';
import changeHandler from './change-handler';
import { getSettings, putSettings } from './settings';

declare global {
  var localDb: any;
  var db: any;
  var dbReady: boolean;
}

export const initDb = async () => {
  console.log({ caller: 'initDb' });

  if (globalThis.localDb === undefined) {
    globalThis.dbReady = false;

    await openDatabases();

    // console.log({ caller: 'initDb / before db.changes' });

    const changes = db
      .changes({ since: 'now', live: true, include_docs: false })
      .on('change', changeHandler)
      .on('complete', (info: any) => {
        console.log({ caller: 'changes / complete', info });
      })
      .on('error', (error: any) => {
        console.log({ caller: 'changes / complete', error });
      });

    const localDbChanges = localDb
      .changes({ since: 'now', live: true, include_docs: false })
      .on('change', changeHandler)
      .on('complete', (info: any) => {
        console.log({ caller: 'localDb changes / complete', info });
      })
      .on('error', (error: any) => {
        console.log({ caller: 'localDb changes / complete', error });
      });

    // console.log({ caller: 'initDb / back from db.changes', changes });

    //  changes.cancel();

    globalThis.dbReady = true;
  }
};
