import { equals, Extent } from 'ol/extent';
import VectorTileSource from 'ol/source/VectorTile.js';

interface Options {
  source: VectorTileSource;
}

class ClusterableVectorTileSourceProxy {
  extent?: Extent;
  resolution?: number;
  featuresSent: boolean = false;
  proxy?: typeof Proxy;

  overloads = {
    loadFeatures: (target: VectorTileSource) => {
      return (extent: Extent, resolution: number) => {
        this.resolution = resolution;
        if (this.extent === undefined || !equals(this.extent, extent)) {
          this.extent = extent;
        }
        if (!this.featuresSent) {
          target.dispatchEvent('change');
        }
        this.featuresSent = false;
        //   console.log({
        //     caller: 'clusterableVectorTileSourceProxy',
        //     overload: 'loadFeatures',
        //     extent,
        //     resolution,
        //     projection,
        //     target: target,
        //     this,
        //   });
      };
    },

    getFeatures: (target: VectorTileSource) => {
      return () => {
        const result =
          this.extent !== undefined
            ? target.getFeaturesInExtent(this.extent)
            : [];
        this.featuresSent = true;
        //   console.log({
        //     caller: 'clusterableVectorTileSourceProxy',
        //     overload: 'getFeatures',
        //     result,
        //     this,
        //     target,
        //   });
        return result;
      };
    },

    addEventListener: (target: VectorTileSource) => {
      return (type: string, listener: any) => {
        console.log({
          caller: 'clusterableVectorTileSourceProxy',
          overload: 'addEventListener',
          type,
          listener,
          target,
        });
        target.addEventListener(type, listener);
        if (type === 'change') {
          target.addEventListener('tileloadend', listener);
        }
      };
    },
  };

  constructor(options: Options) {
    const { source } = options;
    // @ts-ignore
    this.proxy = new Proxy(source, {
      get: (target, prop, receiver) => {
        let result;
        if (prop in this.overloads) {
          result = this.overloads[prop as keyof typeof this.overloads](target);
        } else {
          result = Reflect.get(target, prop, receiver);
        }
        //   console.log({
        //     caller: 'clusterableVectorTileSourceProxy',
        //     trap: 'get',
        //     target,
        //     prop,
        //     receiver,
        //     result,
        //   });
        return result;
      },
    });
    //@ts-ignore
    return this.proxy;
  }
}

export default ClusterableVectorTileSourceProxy;
