import { createEffect } from 'solid-js';
import { until } from '../lib/async-wait';
import { createWorkerSignal } from '../solid-workers/solid-worker-main';
import { getWorker } from './get-worker';

declare global {
  var dispatcherQueue: { index: number; queue: Map<number, any> };
}

export var worker: any;

export const init = () => {
  console.log({ caller: 'dispatcher-main / init' });
  globalThis.dispatcherQueue = { index: 0, queue: new Map() };
  console.log({ caller: 'dispatcher-main / init', globalThis });

  worker = getWorker();
  console.log({ caller: 'dispatcher-main / init', worker });

  worker.onmessage = (event: any) => {
    const { id, payload } = event.data;
    console.log({
      caller: 'dispatcher-main / message received',
      id,
      payload,
      dispatcherQueue: JSON.parse(JSON.stringify(dispatcherQueue)),
    });
    dispatcherQueue.queue.get(id).callBack(null, payload, id);
    if (!dispatcherQueue.queue.get(id).live) {
      cancelDispatch(id);
    }
  };
};

const [watchDbStatus] = createWorkerSignal({
  provider: 'watchDb',
});
const dbReady = () => {
  console.log({
    caller: 'dispatcher-main / waiting',
    dbStatus: watchDbStatus(),
    dbReady: watchDbStatus()?.opened,
  });
  return watchDbStatus()?.opened;
};

const dispatch = (
  payload: any,
  callBack?: (error: any, result: any, id?: number | undefined) => void,
  live?: boolean
) => {
  console.log({ caller: 'dispatcher-main / dispatch', payload });
  if (worker === undefined) {
    init();
  }
  until(dbReady, 1000).then(() => {
    // Wait until databases have been created by health.ts
    if (callBack === undefined) {
      /** If a callback function is not provided, return a promise */
      return new Promise((resolve, reject) => {
        dispatch(payload, (error, result) => {
          if (error) {
            reject(error);
          } else {
            resolve(result);
          }
        });
      });
    }
    /** Otherwise, use the callback function */
    dispatcherQueue.queue.set(dispatcherQueue.index, { callBack, live });
    const message = {
      id: dispatcherQueue.index++,
      payload: payload,
    };
    worker.postMessage(message);
    console.log({
      caller: 'dispatcher-main / message sent',
      message,
      dispatcherQueue,
    });
  });
};

export const cancelDispatch = (id: number) => {
  dispatcherQueue.queue.delete(id);
};

export default dispatch;
